package Net::Kubernetes::Resource::Spec;
$Net::Kubernetes::Resource::Spec::VERSION = '0.06';
use Moose::Role;

=head1 NAME

Net::Kubernetes::Resource::Spec

=head1 VERSION

version 0.06

=cut


has spec => (
	is       => 'rw',
	isa      => 'HashRef',
	required => 1
);

return 42;
