package Net::Kubernetes::Resource::Service;
$Net::Kubernetes::Resource::Service::VERSION = '0.06';
use Moose;

=head1 NAME

Net::Kubernetes::Resource::Service

=head1 VERSION

version 0.06

=cut

extends 'Net::Kubernetes::Resource';

with 'Net::Kubernetes::Resource::State';
with 'Net::Kubernetes::Resource::Spec';

return 42;
