package Net::Kubernetes::Exception;
$Net::Kubernetes::Exception::VERSION = '0.06';
use Moose;

=head1 NAME

Net::Kubernetes::Exception

=head1 VERSION

version 0.06

=cut

extends "Throwable::Error";


has code => (
	is       => 'ro',
	isa      => 'Num',
	required => 1,
);

return 42;
